<?php
namespace nur\sery\file\csv;

use nur\sery\ext\spreadsheet\wsutils;
use nur\sery\os\path;
use nur\sery\web\http;
use PhpOffice\PhpSpreadsheet\Cell\IValueBinder;
use PhpOffice\PhpSpreadsheet\Cell\StringValueBinder;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use PhpOffice\PhpSpreadsheet\Writer\Ods;
use PhpOffice\PhpSpreadsheet\Writer\Xls;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

/**
 * Class SsBuilder: construction d'une feuille de calcul, pour envoi à
 * l'utilisateur
 */
class SsBuilder extends AbstractBuilder {
  use TAbstractBuilder;

  function __construct(?string $output, ?array $params=null) {
    parent::__construct($output, $params);
    $this->ss = new Spreadsheet();
    $this->valueBinder = new StringValueBinder();
    $this->setWsname($params["wsname"] ?? null);
  }

  protected Spreadsheet $ss;

  protected IValueBinder $valueBinder;

  protected ?Worksheet $ws;

  protected int $nrow;

  const STYLE_ROW = 0, STYLE_HEADER = 1;

  protected int $rowStyle;

  function setWsname(?string $wsname): void {
    $ss = $this->ss;
    $this->ws = null;
    $this->nrow = 0;
    $this->rowStyle = self::STYLE_ROW;
    if ($wsname === null) {
      $ws = $ss->getActiveSheet();
    } else {
      $ws = $ss->getSheetByName($wsname);
    }
    if ($ws === null) {
      $ws = $ss->createSheet()->setTitle($wsname);
      $this->wroteHeaders = false;
    } else {
      $maxRow = wsutils::compute_max_coords($ws)[1];
      $this->nrow = $maxRow - 1;
      $this->wroteHeaders = $maxRow > 1;
    }
    $this->ws = $ws;
  }

  function _write(array $row): void {
    $ws = $this->ws;
    $styleHeader = $this->rowStyle === self::STYLE_HEADER;
    $nrow = ++$this->nrow;
    $ncol = 1;
    foreach ($row as $col) {
      $ws->getCellByColumnAndRow($ncol++, $nrow)->setValue($col, $this->valueBinder);
    }
    if ($styleHeader) {
      $ws->getStyle("$nrow:$nrow")->getFont()->setBold(true);
    }
  }

  function writeHeaders(?array $headers=null): void {
    $this->rowStyle = self::STYLE_HEADER;
    parent::writeHeaders($headers);
    $this->rowStyle = self::STYLE_ROW;
  }

  function _sendContentType(): void {
    switch (path::ext($this->output)) {
    case ".ods":
      $contentType = "application/vnd.oasis.opendocument.spreadsheet";
      break;
    case ".xls":
      $contentType = "application/vnd.ms-excel";
      break;
    case ".xlsx":
    default:
      $contentType = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
      break;
    }
    http::content_type($contentType);
  }

  function _sendFile(): int {
    switch (path::ext($this->output)) {
    case ".ods":
      $writer = new Ods($this->ss);
      break;
    case ".xls":
      $writer = new Xls($this->ss);
      break;
    case ".xlsx":
    default:
      $writer = new Xlsx($this->ss);
      break;
    }
    $this->rewind();
    $writer->save($this->getResource());
    $this->rewind();
    $this->sendHeaders();
    return $this->fpassthru();
  }
}
